@@ -2,7 +2,7 @@ from __future__ import absolute_import, unicode_literals
 
 from django.db import models
 from django.utils.safestring import mark_safe
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 
 class RawContent(models.Model):
@@ -13,12 +13,12 @@ class RawContent(models.Model):
     snippets too.
     """
 
-    text = models.TextField(_('content'), blank=True)
+    text = models.TextField(_("content"), blank=True)
 
     class Meta:
         abstract = True
-        verbose_name = _('raw content')
-        verbose_name_plural = _('raw contents')
+        verbose_name = _("raw content")
+        verbose_name_plural = _("raw contents")
 
     def render(self, **kwargs):
         return mark_safe(self.text)
