@@ -1,4 +1,4 @@
-from __future__ import absolute_import, print_function, unicode_literals
+from __future__ import absolute_import, unicode_literals
 
 import os
 
@@ -12,11 +12,11 @@ class Command(NoArgsCommand):
     help = "Prints all orphaned files in the `media/medialibrary` folder"
 
     def handle_noargs(self, **options):
-        mediafiles = list(MediaFile.objects.values_list('file', flat=True))
+        mediafiles = list(MediaFile.objects.values_list("file", flat=True))
 
         # TODO make this smarter, and take MEDIA_ROOT into account
-        for base, dirs, files in os.walk('media/medialibrary'):
+        for base, dirs, files in os.walk("media/medialibrary"):
             for f in files:
                 full = os.path.join(base[6:], f)
                 if force_text(full) not in mediafiles:
-                    print(os.path.join(base, f))
+                    self.stdout.write(os.path.join(base, f))
