@@ -7,7 +7,7 @@
 #
 # ------------------------------------------------------------------------
 
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 
 import json
 import zipfile
@@ -23,7 +23,7 @@ from .models import Category, MediaFile, MediaFileTranslation
 
 
 # ------------------------------------------------------------------------
-export_magic = 'feincms-export-01'
+export_magic = "feincms-export-01"
 
 
 # ------------------------------------------------------------------------
@@ -47,9 +47,9 @@ def import_zipfile(category_id, overwrite, data):
     info = {}
     try:
         info = json.loads(z.comment)
-        if info['export_magic'] == export_magic:
+        if info["export_magic"] == export_magic:
             is_export_file = True
-    except:
+    except Exception:
         pass
 
     # If meta information, do we need to create any categories?
@@ -57,21 +57,21 @@ def import_zipfile(category_id, overwrite, data):
     category_id_map = {}
     if is_export_file:
         for cat in sorted(
-                info.get('categories', []),
-                key=lambda k: k.get('level', 999)):
+            info.get("categories", []), key=lambda k: k.get("level", 999)
+        ):
             new_cat, created = Category.objects.get_or_create(
-                slug=cat['slug'],
-                title=cat['title'])
-            category_id_map[cat['id']] = new_cat
-            if created and cat.get('parent', 0):
-                parent_cat = category_id_map.get(cat.get('parent', 0), None)
+                slug=cat["slug"], title=cat["title"]
+            )
+            category_id_map[cat["id"]] = new_cat
+            if created and cat.get("parent", 0):
+                parent_cat = category_id_map.get(cat.get("parent", 0), None)
                 if parent_cat:
                     new_cat.parent = parent_cat
                     new_cat.save()
 
     count = 0
     for zi in z.infolist():
-        if not zi.filename.endswith('/'):
+        if not zi.filename.endswith("/"):
             bname = os.path.basename(zi.filename)
             if bname and not bname.startswith(".") and "." in bname:
                 fname, ext = os.path.splitext(bname)
@@ -95,36 +95,34 @@ def import_zipfile(category_id, overwrite, data):
                     mf = MediaFile()
                 if overwrite:
                     mf.file.field.upload_to = wanted_dir
-                mf.copyright = info.get('copyright', '')
-                mf.file.save(
-                    target_fname,
-                    ContentFile(z.read(zi.filename)),
-                    save=False)
+                mf.copyright = info.get("copyright", "")
+                mf.file.save(target_fname, ContentFile(z.read(zi.filename)), save=False)
                 mf.save()
 
                 found_metadata = False
                 if is_export_file:
                     try:
-                        for tr in info['translations']:
+                        for tr in info["translations"]:
                             found_metadata = True
-                            mt, mt_created =\
-                                MediaFileTranslation.objects.get_or_create(
-                                    parent=mf, language_code=tr['lang'])
-                            mt.caption = tr['caption']
-                            mt.description = tr.get('description', None)
+                            mt, mt_created = MediaFileTranslation.objects.get_or_create(
+                                parent=mf, language_code=tr["lang"]
+                            )
+                            mt.caption = tr["caption"]
+                            mt.description = tr.get("description", None)
                             mt.save()
 
                         # Add categories
                         mf.categories = (
                             category_id_map[cat_id]
-                            for cat_id in info.get('categories', []))
+                            for cat_id in info.get("categories", [])
+                        )
                     except Exception:
                         pass
 
                 if not found_metadata:
                     mt = MediaFileTranslation()
                     mt.parent = mf
-                    mt.caption = fname.replace('_', ' ')
+                    mt.caption = fname.replace("_", " ")
                     mt.save()
 
                 if category:
@@ -139,10 +137,14 @@ def import_zipfile(category_id, overwrite, data):
 def export_zipfile(site, queryset):
     now = timezone.now()
     zip_name = "export_%s_%04d%02d%02d.zip" % (
-        slugify(site.domain), now.year, now.month, now.day)
+        slugify(site.domain),
+        now.year,
+        now.month,
+        now.day,
+    )
 
     zip_data = open(os.path.join(django_settings.MEDIA_ROOT, zip_name), "w")
-    zip_file = zipfile.ZipFile(zip_data, 'w', allowZip64=True)
+    zip_file = zipfile.ZipFile(zip_data, "w", allowZip64=True)
 
     # Save the used categories in the zip file's global comment
     used_categories = set()
@@ -151,28 +153,36 @@ def export_zipfile(site, queryset):
             used_categories.update(cat.path_list())
 
     info = {
-        'export_magic': export_magic,
-        'categories': [{
-            'id': cat.id,
-            'title': cat.title,
-            'slug': cat.slug,
-            'parent': cat.parent_id or 0,
-            'level': len(cat.path_list()),
-        } for cat in used_categories],
+        "export_magic": export_magic,
+        "categories": [
+            {
+                "id": cat.id,
+                "title": cat.title,
+                "slug": cat.slug,
+                "parent": cat.parent_id or 0,
+                "level": len(cat.path_list()),
+            }
+            for cat in used_categories
+        ],
     }
     zip_file.comment = json.dumps(info)
 
     for mf in queryset:
         ctime = time.localtime(os.stat(mf.file.path).st_ctime)
-        info = json.dumps({
-            'copyright': mf.copyright,
-            'categories': [cat.id for cat in mf.categories.all()],
-            'translations': [{
-                'lang': t.language_code,
-                'caption': t.caption,
-                'description': t.description,
-            } for t in mf.translations.all()],
-        })
+        info = json.dumps(
+            {
+                "copyright": mf.copyright,
+                "categories": [cat.id for cat in mf.categories.all()],
+                "translations": [
+                    {
+                        "lang": t.language_code,
+                        "caption": t.caption,
+                        "description": t.description,
+                    }
+                    for t in mf.translations.all()
+                ],
+            }
+        )
 
         with open(mf.file.path, "r") as file_data:
             zip_info = zipfile.ZipInfo(
@@ -183,10 +193,13 @@ def export_zipfile(site, queryset):
                     ctime.tm_mday,
                     ctime.tm_hour,
                     ctime.tm_min,
-                    ctime.tm_sec))
+                    ctime.tm_sec,
+                ),
+            )
             zip_info.comment = info
             zip_file.writestr(zip_info, file_data.read())
 
     return zip_name
 
+
 # ------------------------------------------------------------------------
