@@ -1,14 +1,15 @@
 from __future__ import absolute_import, unicode_literals
 
+from collections import OrderedDict
+
 from django.http import Http404
 from django.template import Template
-from django.utils.datastructures import SortedDict
 from django.utils.decorators import method_decorator
 from django.views import generic
 from django.views.generic.base import TemplateResponseMixin
 
 from feincms import settings
-from feincms.views.decorators import standalone
+from feincms.content.application.models import standalone
 
 
 class ContentModelMixin(object):
@@ -30,7 +31,7 @@ class ContentModelMixin(object):
         always receives two arguments, the current object and the request.
         """
         if cls.request_processors is None:
-            cls.request_processors = SortedDict()
+            cls.request_processors = OrderedDict()
         cls.request_processors[fn if key is None else key] = fn
 
     @classmethod
@@ -41,7 +42,7 @@ class ContentModelMixin(object):
         request and the response.
         """
         if cls.response_processors is None:
-            cls.response_processors = SortedDict()
+            cls.response_processors = OrderedDict()
         cls.response_processors[fn if key is None else key] = fn
 
     # TODO Implement admin_urlname templatetag protocol
@@ -60,11 +61,6 @@ class ContentModelMixin(object):
         "See app_label"
         return self.__class__.__name__.lower()
 
-    @property
-    def module_name(self):
-        "See app_label"
-        return self.__class__.__name__.lower()
-
 
 class ContentObjectMixin(TemplateResponseMixin):
     """
@@ -83,7 +79,7 @@ class ContentObjectMixin(TemplateResponseMixin):
     context_object_name = None
 
     def handler(self, request, *args, **kwargs):
-        if not hasattr(self.request, '_feincms_extra_context'):
+        if not hasattr(self.request, "_feincms_extra_context"):
             self.request._feincms_extra_context = {}
 
         r = self.run_request_processors()
@@ -125,7 +121,7 @@ class ContentObjectMixin(TemplateResponseMixin):
 
     def get_context_data(self, **kwargs):
         context = self.request._feincms_extra_context
-        context[self.context_object_name or 'feincms_object'] = self.object
+        context[self.context_object_name or "feincms_object"] = self.object
         context.update(kwargs)
         return super(ContentObjectMixin, self).get_context_data(**context)
 
@@ -144,7 +140,7 @@ class ContentObjectMixin(TemplateResponseMixin):
         also return a ``HttpResponse`` for shortcutting the rendering and
         returning that response immediately to the client.
         """
-        if not getattr(self.object, 'request_processors', None):
+        if not getattr(self.object, "request_processors", None):
             return
 
         for fn in reversed(list(self.object.request_processors.values())):
@@ -158,7 +154,7 @@ class ContentObjectMixin(TemplateResponseMixin):
         processors are called to modify the response, eg. for setting cache or
         expiration headers, keeping statistics, etc.
         """
-        if not getattr(self.object, 'response_processors', None):
+        if not getattr(self.object, "response_processors", None):
             return
 
         for fn in self.object.response_processors.values():
@@ -176,8 +172,9 @@ class ContentObjectMixin(TemplateResponseMixin):
         # did any content type successfully end processing?
         successful = False
 
-        for content in self.object.content.all_of_type(tuple(
-                self.object._feincms_content_types_with_process)):
+        for content in self.object.content.all_of_type(
+            tuple(self.object._feincms_content_types_with_process)
+        ):
 
             try:
                 r = content.process(self.request, view=self)
@@ -195,15 +192,17 @@ class ContentObjectMixin(TemplateResponseMixin):
 
             extra_context = self.request._feincms_extra_context
 
-            if (not settings.FEINCMS_ALLOW_EXTRA_PATH
-                    and extra_context.get('extra_path', '/') != '/'
-                    # XXX Already inside application content.  I'm not sure
-                    # whether this fix is really correct...
-                    and not extra_context.get('app_config')):
-                raise Http404(str('Not found (extra_path %r on %r)') % (
-                    extra_context.get('extra_path', '/'),
-                    self.object,
-                ))
+            if (
+                not settings.FEINCMS_ALLOW_EXTRA_PATH
+                and extra_context.get("extra_path", "/") != "/"
+                # XXX Already inside application content.  I'm not sure
+                # whether this fix is really correct...
+                and not extra_context.get("app_config")
+            ):
+                raise Http404(
+                    str("Not found (extra_path %r on %r)")
+                    % (extra_context.get("extra_path", "/"), self.object)
+                )
 
     def finalize_content_types(self, response):
         """
@@ -211,8 +210,9 @@ class ContentObjectMixin(TemplateResponseMixin):
         returns the final response.
         """
 
-        for content in self.object.content.all_of_type(tuple(
-                self.object._feincms_content_types_with_finalize)):
+        for content in self.object.content.all_of_type(
+            tuple(self.object._feincms_content_types_with_finalize)
+        ):
 
             r = content.finalize(self.request, response)
             if r:
