@@ -4,32 +4,27 @@
 
 from __future__ import absolute_import, unicode_literals
 
-import re
+import six
 
-from django.core.cache import cache as django_cache
 from django.core.exceptions import PermissionDenied
-from django.conf import settings as django_settings
 from django.db import models
-from django.db.models import Q, signals
-from django.db.models.loading import get_model
+from django.db.models import Q
 from django.http import Http404
-from django.utils.encoding import python_2_unicode_compatible
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
+
+try:
+    from django.urls import reverse
+except ImportError:
+    from django.core.urlresolvers import reverse
 
 from mptt.models import MPTTModel, TreeManager
 
 from feincms import settings
-from feincms.management.checker import check_database_schema
 from feincms.models import create_base_model
 from feincms.module.mixins import ContentModelMixin
 from feincms.module.page import processors
 from feincms.utils.managers import ActiveAwareContentManagerMixin
-
-from feincms.utils import path_to_cache_key, shorten_string
-
-
-REDIRECT_TO_RE = re.compile(
-    r'^(?P<app_label>\w+).(?P<model_name>\w+):(?P<pk>\d+)$')
+from feincms.utils import shorten_string, match_model_string, get_model_instance
 
 
 # ------------------------------------------------------------------------
@@ -40,8 +35,7 @@ class BasePageManager(ActiveAwareContentManagerMixin, TreeManager):
     """
 
     # The fields which should be excluded when creating a copy.
-    exclude_from_copy = [
-        'id', 'tree_id', 'lft', 'rght', 'level', 'redirect_to']
+    exclude_from_copy = ["id", "tree_id", "lft", "rght", "level", "redirect_to"]
 
     def page_for_path(self, path, raise404=False):
         """
@@ -52,14 +46,13 @@ class BasePageManager(ActiveAwareContentManagerMixin, TreeManager):
             Page.objects.page_for_path(request.path)
         """
 
-        stripped = path.strip('/')
+        stripped = path.strip("/")
 
         try:
-            page = self.active().get(
-                _cached_url='/%s/' % stripped if stripped else '/')
+            page = self.active().get(_cached_url="/%s/" % stripped if stripped else "/")
 
             if not page.are_ancestors_active():
-                raise self.model.DoesNotExist('Parents are inactive.')
+                raise self.model.DoesNotExist("Parents are inactive.")
 
             return page
 
@@ -80,33 +73,24 @@ class BasePageManager(ActiveAwareContentManagerMixin, TreeManager):
         page with url '/photos/album/'.
         """
 
-        paths = ['/']
-        path = path.strip('/')
-
-        # Cache path -> page resolving.
-        # We flush the cache entry on page saving, so the cache should always
-        # be up to date.
-
-        ck = Page.path_to_cache_key(path)
-        page = django_cache.get(ck)
-        if page:
-            return page
+        paths = ["/"]
+        path = path.strip("/")
 
         if path:
-            tokens = path.split('/')
-            paths += [
-                '/%s/' % '/'.join(tokens[:i])
-                for i in range(1, len(tokens) + 1)]
+            tokens = path.split("/")
+            paths += ["/%s/" % "/".join(tokens[:i]) for i in range(1, len(tokens) + 1)]
 
         try:
-            page = self.active().filter(_cached_url__in=paths).extra(
-                select={'_url_length': 'LENGTH(_cached_url)'}
-            ).order_by('-_url_length')[0]
+            page = (
+                self.active()
+                .filter(_cached_url__in=paths)
+                .extra(select={"_url_length": "LENGTH(_cached_url)"})
+                .order_by("-_url_length")[0]
+            )
 
             if not page.are_ancestors_active():
-                raise IndexError('Parents are inactive.')
+                raise IndexError("Parents are inactive.")
 
-            django_cache.set(ck, page)
             return page
 
         except IndexError:
@@ -129,8 +113,7 @@ class BasePageManager(ActiveAwareContentManagerMixin, TreeManager):
 
         return self.in_navigation().filter(parent__isnull=True)
 
-    def for_request(self, request, raise404=False, best_match=False,
-                    path=None):
+    def for_request(self, request, raise404=False, best_match=False, path=None):
         """
         Return a page for the request
 
@@ -144,15 +127,15 @@ class BasePageManager(ActiveAwareContentManagerMixin, TreeManager):
         could be determined.
         """
 
-        if not hasattr(request, '_feincms_page'):
+        if not hasattr(request, "_feincms_page"):
             path = path or request.path_info or request.path
 
             if best_match:
                 request._feincms_page = self.best_match_for_path(
-                    path, raise404=raise404)
+                    path, raise404=raise404
+                )
             else:
-                request._feincms_page = self.page_for_path(
-                    path, raise404=raise404)
+                request._feincms_page = self.page_for_path(path, raise404=raise404)
 
         return request._feincms_page
 
@@ -160,44 +143,67 @@ class BasePageManager(ActiveAwareContentManagerMixin, TreeManager):
 # ------------------------------------------------------------------------
 class PageManager(BasePageManager):
     pass
-PageManager.add_to_active_filters(Q(active=True))
+
+
+PageManager.add_to_active_filters(Q(active=True), key="is_active")
 
 
 # ------------------------------------------------------------------------
-@python_2_unicode_compatible
+@six.python_2_unicode_compatible
 class BasePage(create_base_model(MPTTModel), ContentModelMixin):
-    active = models.BooleanField(_('active'), default=True)
+    active = models.BooleanField(_("active"), default=True)
 
     # structure and navigation
-    title = models.CharField(_('title'), max_length=200, help_text=_(
-        'This title is also used for navigation menu items.'))
+    title = models.CharField(
+        _("title"),
+        max_length=200,
+        help_text=_("This title is also used for navigation menu items."),
+    )
     slug = models.SlugField(
-        _('slug'), max_length=150,
-        help_text=_('This is used to build the URL for this page'))
+        _("slug"),
+        max_length=150,
+        help_text=_("This is used to build the URL for this page"),
+    )
     parent = models.ForeignKey(
-        'self', verbose_name=_('Parent'), blank=True,
-        null=True, related_name='children')
+        "self",
+        verbose_name=_("Parent"),
+        blank=True,
+        on_delete=models.CASCADE,
+        null=True,
+        related_name="children",
+    )
     # Custom list_filter - see admin/filterspecs.py
     parent.parent_filter = True
-    in_navigation = models.BooleanField(_('in navigation'), default=False)
+    in_navigation = models.BooleanField(_("in navigation"), default=False)
     override_url = models.CharField(
-        _('override URL'), max_length=255,
-        blank=True, help_text=_(
-            'Override the target URL. Be sure to include slashes at the '
-            'beginning and at the end if it is a local URL. This '
-            'affects both the navigation and subpages\' URLs.'))
-    redirect_to = models.CharField(
-        _('redirect to'), max_length=255,
+        _("override URL"),
+        max_length=255,
         blank=True,
         help_text=_(
-            'Target URL for automatic redirects'
-            ' or the primary key of a page.'))
+            "Override the target URL. Be sure to include slashes at the "
+            "beginning and at the end if it is a local URL. This "
+            "affects both the navigation and subpages' URLs."
+        ),
+    )
+    redirect_to = models.CharField(
+        _("redirect to"),
+        max_length=255,
+        blank=True,
+        help_text=_(
+            "Target URL for automatic redirects" " or the primary key of a page."
+        ),
+    )
     _cached_url = models.CharField(
-        _('Cached URL'), max_length=255, blank=True,
-        editable=False, default='', db_index=True)
+        _("Cached URL"),
+        max_length=255,
+        blank=True,
+        editable=False,
+        default="",
+        db_index=True,
+    )
 
     class Meta:
-        ordering = ['tree_id', 'lft']
+        ordering = ["tree_id", "lft"]
         abstract = True
 
     objects = PageManager()
@@ -213,12 +219,16 @@ class BasePage(create_base_model(MPTTModel), ContentModelMixin):
         if not self.pk:
             return False
 
+        # No need to hit DB if page itself is inactive
+        if not self.active:
+            return False
+
         pages = self.__class__.objects.active().filter(
-            tree_id=self.tree_id,
-            lft__lte=self.lft,
-            rght__gte=self.rght)
+            tree_id=self.tree_id, lft__lte=self.lft, rght__gte=self.rght
+        )
         return pages.count() > self.level
-    is_active.short_description = _('is active')
+
+    is_active.short_description = _("is active")
 
     def are_ancestors_active(self):
         """
@@ -236,8 +246,9 @@ class BasePage(create_base_model(MPTTModel), ContentModelMixin):
         Title shortened for display.
         """
         return shorten_string(self.title)
-    short_title.admin_order_field = 'title'
-    short_title.short_description = _('title')
+
+    short_title.admin_order_field = "title"
+    short_title.short_description = _("title")
 
     def __init__(self, *args, **kwargs):
         super(BasePage, self).__init__(*args, **kwargs)
@@ -258,17 +269,13 @@ class BasePage(create_base_model(MPTTModel), ContentModelMixin):
         if self.override_url:
             self._cached_url = self.override_url
         elif self.is_root_node():
-            self._cached_url = '/%s/' % self.slug
+            self._cached_url = "/%s/" % self.slug
         else:
-            self._cached_url = '%s%s/' % (self.parent._cached_url, self.slug)
+            self._cached_url = "%s%s/" % (self.parent._cached_url, self.slug)
 
         cached_page_urls[self.id] = self._cached_url
         super(BasePage, self).save(*args, **kwargs)
 
-        # Okay, we have changed the page -- remove the old stale entry from the
-        # cache
-        self.invalidate_cache()
-
         # If our cached URL changed we need to update all descendants to
         # reflect the changes. Since this is a very expensive operation
         # on large sites we'll check whether our _cached_url actually changed
@@ -276,75 +283,55 @@ class BasePage(create_base_model(MPTTModel), ContentModelMixin):
         if self._cached_url == self._original_cached_url:
             return
 
-        pages = self.get_descendants().order_by('lft')
+        pages = self.get_descendants().order_by("lft")
 
         for page in pages:
             if page.override_url:
                 page._cached_url = page.override_url
             else:
                 # cannot be root node by definition
-                page._cached_url = '%s%s/' % (
+                page._cached_url = "%s%s/" % (
                     cached_page_urls[page.parent_id],
-                    page.slug)
+                    page.slug,
+                )
 
             cached_page_urls[page.id] = page._cached_url
             super(BasePage, page).save()  # do not recurse
+
     save.alters_data = True
 
     def delete(self, *args, **kwargs):
         if not settings.FEINCMS_SINGLETON_TEMPLATE_DELETION_ALLOWED:
             if self.template.singleton:
-                raise PermissionDenied(_(
-                    'This %(page_class)s uses a singleton template, and '
-                    'FEINCMS_SINGLETON_TEMPLATE_DELETION_ALLOWED=False' % {
-                        'page_class': self._meta.verbose_name}))
+                raise PermissionDenied(
+                    _(
+                        "This %(page_class)s uses a singleton template, and "
+                        "FEINCMS_SINGLETON_TEMPLATE_DELETION_ALLOWED=False"
+                        % {"page_class": self._meta.verbose_name}
+                    )
+                )
         super(BasePage, self).delete(*args, **kwargs)
-        self.invalidate_cache()
+
     delete.alters_data = True
 
-    # Remove the page from the url-to-page cache
-    def invalidate_cache(self):
-        ck = self.path_to_cache_key(self._original_cached_url)
-        django_cache.delete(ck)
-        ck = self.path_to_cache_key(self._cached_url)
-        django_cache.delete(ck)
-
-    @models.permalink
     def get_absolute_url(self):
         """
         Return the absolute URL of this page.
         """
         # result url never begins or ends with a slash
-        url = self._cached_url.strip('/')
+        url = self._cached_url.strip("/")
         if url:
-            return ('feincms_handler', (url,), {})
-        return ('feincms_home', (), {})
+            return reverse("feincms_handler", args=(url,))
+        return reverse("feincms_home")
 
     def get_navigation_url(self):
         """
         Return either ``redirect_to`` if it is set, or the URL of this page.
         """
 
-        # :-( maybe this could be cleaned up a bit?
-        if not self.redirect_to or REDIRECT_TO_RE.match(self.redirect_to):
-            return self._cached_url
-        return self.redirect_to
-
-    cache_key_components = [
-        lambda p: getattr(django_settings, 'SITE_ID', 0),
-        lambda p: p._django_content_type.id,
-        lambda p: p.id,
-    ]
-
-    def cache_key(self):
-        """
-        Return a string that may be used as cache key for the current page.
-        The cache_key is unique for each content type and content instance.
-
-        This function is here purely for your convenience. FeinCMS itself
-        does not use it in any way.
-        """
-        return '-'.join(str(fn(self)) for fn in self.cache_key_components)
+        if self.redirect_to:
+            return self.get_redirect_to_target()
+        return self._cached_url
 
     def etag(self, request):
         """
@@ -365,75 +352,56 @@ class BasePage(create_base_model(MPTTModel), ContentModelMixin):
         """
         return None
 
-    def get_redirect_to_target(self, request):
+    def get_redirect_to_page(self):
         """
         This might be overriden/extended by extension modules.
         """
 
         if not self.redirect_to:
-            return ''
+            return None
 
         # It might be an identifier for a different object
-        match = REDIRECT_TO_RE.match(self.redirect_to)
+        whereto = match_model_string(self.redirect_to)
+        if not whereto:
+            return None
 
-        # It's not, oh well.
-        if not match:
+        return get_model_instance(*whereto)
+
+    def get_redirect_to_target(self, request=None):
+        """
+        This might be overriden/extended by extension modules.
+        """
+
+        target_page = self.get_redirect_to_page()
+        if target_page is None:
             return self.redirect_to
 
-        matches = match.groupdict()
-        model = get_model(matches['app_label'], matches['model_name'])
-
-        if not model:
-            return self.redirect_to
-
-        try:
-            instance = model._default_manager.get(pk=int(matches['pk']))
-            return instance.get_absolute_url()
-        except models.ObjectDoesNotExist:
-            pass
-
-        return self.redirect_to
+        return target_page.get_absolute_url()
 
     @classmethod
-    def path_to_cache_key(cls, path):
-        prefix = "%s-FOR-URL" % cls.__name__.upper()
-        return path_to_cache_key(path.strip('/'), prefix=prefix)
-
-    @classmethod
-    def register_default_processors(cls, frontend_editing=False):
+    def register_default_processors(cls):
         """
         Register our default request processors for the out-of-the-box
         Page experience.
         """
         cls.register_request_processor(
-            processors.redirect_request_processor, key='redirect')
+            processors.redirect_request_processor, key="redirect"
+        )
         cls.register_request_processor(
-            processors.extra_context_request_processor, key='extra_context')
-
-        if frontend_editing:
-            cls.register_request_processor(
-                processors.frontendediting_request_processor,
-                key='frontend_editing')
-            cls.register_response_processor(
-                processors.frontendediting_response_processor,
-                key='frontend_editing')
+            processors.extra_context_request_processor, key="extra_context"
+        )
 
 
 # ------------------------------------------------------------------------
 class Page(BasePage):
     class Meta:
-        ordering = ['tree_id', 'lft']
-        verbose_name = _('page')
-        verbose_name_plural = _('pages')
+        ordering = ["tree_id", "lft"]
+        verbose_name = _("page")
+        verbose_name_plural = _("pages")
+        app_label = "page"
         # not yet # permissions = (("edit_page", _("Can edit page metadata")),)
 
-Page.register_default_processors(
-    frontend_editing=settings.FEINCMS_FRONTEND_EDITING)
 
-if settings.FEINCMS_CHECK_DATABASE_SCHEMA:
-    signals.post_syncdb.connect(
-        check_database_schema(Page, __name__),
-        weak=False)
+Page.register_default_processors()
 
 # ------------------------------------------------------------------------
-# ------------------------------------------------------------------------
