@@ -1,11 +1,14 @@
 from __future__ import absolute_import, print_function, unicode_literals
 
+import logging
 import re
 import sys
 
 from django.conf import settings as django_settings
 from django.http import Http404, HttpResponseRedirect
-from django.utils.cache import add_never_cache_headers
+
+
+logger = logging.getLogger(__name__)
 
 
 def redirect_request_processor(page, request):
@@ -15,8 +18,14 @@ def redirect_request_processor(page, request):
     """
     target = page.get_redirect_to_target(request)
     if target:
-        if request._feincms_extra_context.get('extra_path', '/') == '/':
+        extra_path = request._feincms_extra_context.get("extra_path", "/")
+        if extra_path == "/":
             return HttpResponseRedirect(target)
+        logger.debug(
+            "Page redirect on '%s' not taken because extra path '%s' present",
+            page.get_absolute_url(),
+            extra_path,
+        )
         raise Http404()
 
 
@@ -24,57 +33,24 @@ def extra_context_request_processor(page, request):
     """
     Fills ``request._feincms_extra_context`` with a few useful variables.
     """
-    request._feincms_extra_context.update({
-        # XXX This variable name isn't accurate anymore.
-        'in_appcontent_subpage': False,
-        'extra_path': '/',
-    })
+    request._feincms_extra_context.update(
+        {
+            # XXX This variable name isn't accurate anymore.
+            "in_appcontent_subpage": False,
+            "extra_path": "/",
+        }
+    )
 
     url = page.get_absolute_url()
     if request.path != url:
-        request._feincms_extra_context.update({
-            'in_appcontent_subpage': True,
-            'extra_path': re.sub(
-                '^' + re.escape(url.rstrip('/')),
-                '',
-                request.path,
-            ),
-        })
-
-
-def frontendediting_request_processor(page, request):
-    """
-    Sets the frontend editing state in the cookie depending on the
-    ``frontend_editing`` GET parameter and the user's permissions.
-    """
-    if 'frontend_editing' not in request.GET:
-        return
-
-    response = HttpResponseRedirect(request.path)
-    if request.user.has_module_perms('page'):
-        try:
-            enable_fe = int(request.GET['frontend_editing']) > 0
-        except ValueError:
-            enable_fe = False
-
-        if enable_fe:
-            response.set_cookie(str('frontend_editing'), enable_fe)
-        else:
-            response.delete_cookie(str('frontend_editing'))
-
-    # Redirect to cleanup URLs
-    return response
-
-
-def frontendediting_response_processor(page, request, response):
-    # Add never cache headers in case frontend editing is active
-    if (hasattr(request, 'COOKIES')
-            and request.COOKIES.get('frontend_editing', False)):
-
-        if hasattr(response, 'add_post_render_callback'):
-            response.add_post_render_callback(add_never_cache_headers)
-        else:
-            add_never_cache_headers(response)
+        request._feincms_extra_context.update(
+            {
+                "in_appcontent_subpage": True,
+                "extra_path": re.sub(
+                    "^" + re.escape(url.rstrip("/")), "", request.path
+                ),
+            }
+        )
 
 
 def etag_request_processor(page, request):
@@ -90,6 +66,11 @@ def etag_request_processor(page, request):
         This is a dummy class with enough behaviour of HttpResponse so we
         can use the condition decorator without too much pain.
         """
+
+        @property
+        def headers(self):
+            return self
+
         def has_header(page, what):
             return False
 
@@ -113,7 +94,8 @@ def etag_request_processor(page, request):
     # the handler if processing is to continue and a non-DummyResponse
     # (should be a "304 not modified") if the etag matches.
     rsp = condition(etag_func=etagger, last_modified_func=lastmodifier)(
-        dummy_response_handler)(request, page)
+        dummy_response_handler
+    )(request, page)
 
     # If dummy then don't do anything, if a real response, return and
     # thus shortcut the request processing.
@@ -129,7 +111,7 @@ def etag_response_processor(page, request, response):
     """
     etag = page.etag(request)
     if etag is not None:
-        response['ETag'] = '"' + etag + '"'
+        response["ETag"] = '"' + etag + '"'
 
 
 def debug_sql_queries_response_processor(verbose=False, file=sys.stderr):
@@ -151,13 +133,16 @@ def debug_sql_queries_response_processor(verbose=False, file=sys.stderr):
     def processor(page, request, response):
         from django.db import connection
 
-        print_sql = lambda x: x
         try:
             import sqlparse
-            print_sql = lambda x: sqlparse.format(
-                x, reindent=True, keyword_case='upper')
-        except:
-            pass
+
+            def print_sql(x):
+                return sqlparse.format(x, reindent=True, keyword_case="upper")
+
+        except Exception:
+
+            def print_sql(x):
+                return x
 
         if verbose:
             print("-" * 60, file=file)
@@ -166,9 +151,10 @@ def debug_sql_queries_response_processor(verbose=False, file=sys.stderr):
         for q in connection.queries:
             i += 1
             if verbose:
-                print("%d : [%s]\n%s\n" % (
-                    i, q['time'], print_sql(q['sql'])), file=file)
-            time += float(q['time'])
+                print(
+                    "%d : [%s]\n%s\n" % (i, q["time"], print_sql(q["sql"])), file=file
+                )
+            time += float(q["time"])
 
         print("-" * 60, file=file)
         print("Total: %d queries, %.3f ms" % (i, time), file=file)
