@@ -1,17 +1,23 @@
 from __future__ import absolute_import, unicode_literals
 
 from django import template
-from django.core.urlresolvers import NoReverseMatch
 from django.template import TemplateSyntaxError
 from django.template.defaulttags import kwarg_re
 from django.utils.encoding import smart_str
 
+try:
+    from django.urls import NoReverseMatch
+except ImportError:
+    from django.core.urlresolvers import NoReverseMatch
+
 from feincms.content.application.models import (
-    ApplicationContent, app_reverse as do_app_reverse)
+    ApplicationContent,
+    app_reverse as do_app_reverse,
+)
 from feincms.templatetags.feincms_tags import _render_content
+
 # backwards compatibility import
-from feincms.templatetags.fragment_tags import (
-    fragment, get_fragment, has_fragment)
+from feincms.templatetags.fragment_tags import fragment, get_fragment, has_fragment
 
 
 register = template.Library()
@@ -35,10 +41,21 @@ def feincms_render_region_appcontent(page, region, request):
             {% feincms_render_region_appcontent feincms_page "main" request %}
         {% endif %}
     """
-    return ''.join(
+    return "".join(
         _render_content(content, request=request)
         for content in page.content.all_of_type(ApplicationContent)
-        if content.region == region)
+        if content.region == region
+    )
+
+
+def _current_app(context):
+    try:
+        return context.request.current_app
+    except AttributeError:
+        try:
+            return context.request.resolver_match.namespace
+        except AttributeError:
+            return getattr(context, "current_app", None)
 
 
 class AppReverseNode(template.Node):
@@ -51,24 +68,31 @@ class AppReverseNode(template.Node):
 
     def render(self, context):
         args = [arg.resolve(context) for arg in self.args]
-        kwargs = dict([
-            (smart_str(k, 'ascii'), v.resolve(context))
-            for k, v in self.kwargs.items()])
+        kwargs = dict(
+            [
+                (smart_str(k, "ascii"), v.resolve(context))
+                for k, v in self.kwargs.items()
+            ]
+        )
         view_name = self.view_name.resolve(context)
         urlconf = self.urlconf.resolve(context)
 
         try:
             url = do_app_reverse(
-                view_name, urlconf, args=args, kwargs=kwargs,
-                current_app=context.current_app)
+                view_name,
+                urlconf,
+                args=args,
+                kwargs=kwargs,
+                current_app=_current_app(context),
+            )
         except NoReverseMatch:
             if self.asvar is None:
                 raise
-            url = ''
+            url = ""
 
         if self.asvar:
             context[self.asvar] = url
-            return ''
+            return ""
         else:
             return url
 
@@ -106,14 +130,15 @@ def app_reverse(parser, token):
     if len(bits) < 3:
         raise TemplateSyntaxError(
             "'%s' takes at least two arguments"
-            " (path to a view and a urlconf)" % bits[0])
+            " (path to a view and a urlconf)" % bits[0]
+        )
     viewname = parser.compile_filter(bits[1])
     urlconf = parser.compile_filter(bits[2])
     args = []
     kwargs = {}
     asvar = None
     bits = bits[3:]
-    if len(bits) >= 2 and bits[-2] == 'as':
+    if len(bits) >= 2 and bits[-2] == "as":
         asvar = bits[-1]
         bits = bits[:-2]
 
@@ -121,8 +146,7 @@ def app_reverse(parser, token):
         for bit in bits:
             match = kwarg_re.match(bit)
             if not match:
-                raise TemplateSyntaxError(
-                    "Malformed arguments to app_reverse tag")
+                raise TemplateSyntaxError("Malformed arguments to app_reverse tag")
             name, value = match.groups()
             if name:
                 kwargs[name] = parser.compile_filter(value)
