@@ -1,14 +1,16 @@
 from __future__ import absolute_import, unicode_literals
 
-import django
+from collections import OrderedDict
+
 from django import template
+from django.contrib.auth import get_permission_codename
 
 
 register = template.Library()
 
 
-@register.filter
-def post_process_fieldsets(fieldset):
+@register.simple_tag(takes_context=True)
+def post_process_fieldsets(context, fieldset):
     """
     Removes a few fields from FeinCMS admin inlines, those being
     ``id``, ``DELETE`` and ``ORDER`` currently.
@@ -21,7 +23,7 @@ def post_process_fieldsets(fieldset):
         return fieldset
 
     fields_to_include = set(fieldset.form.fields.keys())
-    for f in ('id', 'DELETE', 'ORDER'):
+    for f in ("id", "DELETE", "ORDER"):
         fields_to_include.discard(f)
 
     def _filter_recursive(fields):
@@ -44,21 +46,38 @@ def post_process_fieldsets(fieldset):
     for f in fields_to_include:
         new_fields.append(f)
 
+    if context.get("request"):
+        new_fields.extend(
+            list(
+                fieldset.model_admin.get_readonly_fields(
+                    context.get("request"), context.get("original")
+                )
+            )
+        )
+
     fieldset.fields = new_fields
-    return fieldset
+    return ""
 
 
-@register.assignment_tag
-def is_popup_var():
+@register.inclusion_tag(
+    "admin/feincms/content_type_selection_widget.html", takes_context=True
+)
+def show_content_type_selection_widget(context, region):
     """
-    Django 1.6 requires _popup=1 for raw id field popups, earlier versions
-    require pop=1.
-
-    The explicit version check is a bit ugly, but works well.
-
-    (Wrong parameters aren't simply ignored by django.contrib.admin, the
-    change list actively errors out by redirecting to ?e=1)
+    {% show_content_type_selection_widget region %}
     """
-    if django.VERSION < (1, 6):
-        return 'pop=1'
-    return '_popup=1'
+    user = context["request"].user
+    types = OrderedDict({None: []})
+
+    for ct in region._content_types:
+        # Skip cts that we shouldn't be adding anyway
+        opts = ct._meta
+        perm = opts.app_label + "." + get_permission_codename("add", opts)
+        if not user.has_perm(perm):
+            continue
+
+        types.setdefault(getattr(ct, "optgroup", None), []).append(
+            (ct.__name__.lower, ct._meta.verbose_name)
+        )
+
+    return {"types": types}
