@@ -14,7 +14,7 @@ For example::
 
     def lookup_tags(item_qs):
         item_pks = [item.pk for item in item_qs]
-        m2mfield = Item._meta.get_field_by_name('tags')[0]
+        m2mfield = Item._meta.get_field('tags')[0]
         tags_for_item = Tag.objects.filter(
             item__in = item_pks
         ).extra(select = {
@@ -83,6 +83,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 from __future__ import absolute_import, unicode_literals
 
+import django
 from django.db import models
 
 
@@ -90,9 +91,10 @@ class TransformQuerySet(models.query.QuerySet):
     def __init__(self, *args, **kwargs):
         super(TransformQuerySet, self).__init__(*args, **kwargs)
         self._transform_fns = []
+        self._orig_iterable_class = getattr(self, "_iterable_class", None)
 
-    def _clone(self, klass=None, setup=False, **kw):
-        c = super(TransformQuerySet, self)._clone(klass, setup, **kw)
+    def _clone(self, *args, **kwargs):
+        c = super(TransformQuerySet, self)._clone(*args, **kwargs)
         c._transform_fns = self._transform_fns[:]
         return c
 
@@ -101,26 +103,44 @@ class TransformQuerySet(models.query.QuerySet):
         c._transform_fns.extend(fn)
         return c
 
-    def iterator(self):
-        result_iter = super(TransformQuerySet, self).iterator()
-        if self._transform_fns:
+    if django.VERSION < (1, 11):
+
+        def iterator(self):
+            result_iter = super(TransformQuerySet, self).iterator()
+
+            if not self._transform_fns:
+                return result_iter
+
+            if (
+                getattr(self, "_iterable_class", None) != self._orig_iterable_class
+            ):  # noqa
+                # Do not process the result of values() and values_list()
+                return result_iter
+
             results = list(result_iter)
             for fn in self._transform_fns:
                 fn(results)
             return iter(results)
-        return result_iter
+
+    else:
+
+        def _fetch_all(self):
+            super(TransformQuerySet, self)._fetch_all()
+            if (
+                getattr(self, "_iterable_class", None) == self._orig_iterable_class
+            ):  # noqa
+                for fn in self._transform_fns:
+                    fn(self._result_cache)
 
 
-if hasattr(models.Manager, 'from_queryset'):
+if hasattr(models.Manager, "from_queryset"):
     TransformManager = models.Manager.from_queryset(TransformQuerySet)
 
 else:
+
     class TransformManager(models.Manager):
         def get_queryset(self):
             return TransformQuerySet(self.model, using=self._db)
 
-        def get_query_set(self):
-            return TransformQuerySet(self.model, using=self._db)
-
         def transform(self, *fn):
-            return self.get_query_set().transform(*fn)
+            return self.get_queryset().transform(*fn)
